#!/usr/bin/env php
<?php
require('ecologram.php');

$cron_lock_file = '.cron.lock';
$job_start_time = time();
$timeout = isset($eg->config['cron_timeout']) ? $eg->config['cron_timeout'] : 60;

if(file_exists($cron_lock_file)){
  $time_diff = time()-filemtime($cron_lock_file);
  echo "Cron already running since $time_diff seconds.\n";
  if($time_diff > $timeout){
    echo "\nFlush overpassed lock file.\n";
    unlink($cron_lock_file);
  }
  else{
    exit;
  }
}

fopen($cron_lock_file, 'a');

if(!CLI){
    $eg->stream_headers();

    $token = filter_input(INPUT_GET, 'token');
    $verbose = filter_input(INPUT_GET, 'v', FILTER_SANITIZE_NUMBER_INT);
    if(!is_numeric($verbose)){
        $verbose = 0;
    }

    if ($token != $eg->config['cron_token']) {
        _e('Bad credentials');
        exit;
    }
}
else{
  $verbose = 0;
  array_shift($argv);
  foreach ($argv as $arg) {
      $arg_kv = explode('=', $arg);
      if($arg_kv[0]=='v'){
          $verbose = $arg_kv[1];
      }
  }
}

$verbosity = array(
  '-1'=>'Muted',
  0=>'Quiet',
  1=>'Verbose',
  2=>'Blabbermouth',
);

$verboses = array_keys($verbosity);
sort($verboses);
$max_verbose = array_pop($verboses);
$verbose = min($verbose, $max_verbose);

echo"A U T O C R O N\n\nVerbosity: ".$verbosity[$verbose]."\n\n";
if($verbose>-1){
echo "\nInitialize tasks\n"
   . "\n________________\n";
}

$timeout = isset($eg->config['cron_timeout']) ? $eg->config['cron_timeout'] : 60;
$max_sessions = isset($eg->config['cron_max_sessions']) ? $eg->config['cron_max_sessions'] : 30;

//create the multiple cURL handle
$mh = curl_multi_init();

$active_sessions = $eg->list_sessions(array(
  'active' => true,
  'orderby' => 'date_fetched ASC',
  'limit' => $max_sessions,
  'return' => array(
    'name',
    'token',
    'id',
  ),
));
$sessions_count = count($active_sessions);

$eg->log("Autocron launched  for {$sessions_count} sessions", "cron");


$queue = array();
function enqueue_url($url){
    global $timeout, $queue, $mh, $verbose;
    $i = count($queue);
    $queue[$i] = curl_init();
    curl_setopt($queue[$i], CURLOPT_URL, $url);
    curl_setopt($queue[$i], CURLOPT_HEADER, true);
    curl_setopt($queue[$i], CURLOPT_RETURNTRANSFER, true);
    curl_setopt($queue[$i], CURLOPT_SSL_VERIFYPEER, false);
    curl_setopt($queue[$i], CURLOPT_SSL_VERIFYHOST, false);
    curl_setopt($queue[$i], CURLOPT_TIMEOUT, $timeout);
    curl_multi_add_handle($mh, $queue[$i]); //add the handles
    if($verbose > 0){
        echo "- - Enqueue Job #$i\n";
    }
}

// create both cURL resources
foreach ($active_sessions as $i => $session) {
    $networks = $eg->get_networks($session->id);
    foreach ($networks as $net_name=>$network){
        if($network['status']==1){
            $url = $eg->full_base_url . 'cron.php?' . http_build_query(array(
                's' => $session->name,
                't' => $session->token,
                'n' => $net_name,
            ));
            if($verbose > 0){
                echo "- Load $session->name : $net_name\n";
            }
            enqueue_url($url);
        }
    }


}

$active = null;

if($verbose > -1){
printf ("\nLaunche %d CRON tasks\n"
   . "\n__________________\n", count($queue));
}

do {
    curl_multi_exec($mh, $active);
    if($verbose > 0){
        echo "\n> $active @ ".microtime();
    }
    curl_multi_select($mh);
}
while ($active > 0);

if($verbose > -1){
    echo "\n\nRetreive datas from tasks\n"
     . "_________________________\n";
}

foreach ($queue as $id=>$session_curl) {
    if($verbose > 0){
        echo "\n-------------\n";
        echo "\nCode: " . curl_getinfo($session_curl, CURLINFO_HTTP_CODE);
        echo "\n";
    }
    $local_content = curl_multi_getcontent($session_curl);
    if($verbose > 1){
        echo substr($local_content, strpos($local_content, '...................'));
    }

    curl_multi_remove_handle($mh, $session_curl);
    if($verbose > 0){
        echo "\n- Unload $id\n\n";
    }
}

curl_multi_close($mh);

$diff = time()-$job_start_time;

$eg->log("Autocron done  for {$sessions_count} sessions in {$diff} seconds", "cron");

echo "\n-----\n\nDone in $diff seconds. Bye!\n";
unlink($cron_lock_file);
exit;
