<?php

class Model_Manager{
    public $table;
    private $DB;

    // To be overriden
    public $entity_class='';
    public $table_name='';
    public $attributes = [
        'id',
    ];
    private $cache = [];

    function __construct(){
        $this->table = SYN()->config['db_prefix'].$this->table_name;
        $this->DB = SYN()->mysqli;
    }

    function parse_item($item, $key){
        $classname = $this->entity_class;
        return new $classname((array) $item);
    }

    function where_attr($key, $value){
        return " AND {$this->table_name}.`{$key}`='{$value}'";
    }

    function where($atts){
        $where = SYN()->db_where().' 1 ';
        if(is_numeric($atts)){
            $where.=" AND `id`={$atts['id']}";
            return $where;
        }
        $atts=parse_params(
            $atts,
            array_fill_keys(
                array_keys($this->attributes),
                ''
            )
        );
        if(!empty($atts['id'])){
            $where.=" AND `id`='".abs($atts['id'])."'";
        }
        else{
            foreach ($this->attributes as $attribute) {
                if(!empty($atts[$attribute])) $where.=$this->where_attr($attribute, $atts[$attribute]);
            }
        }
        return $where;
    }

    function get($atts){
        $stat_name = 'get_'.md5(json_encode($atts));
        if (isset($this->cache[$stat_name])) {
            return $this->cache[$stat_name];
        }
        global $wpdb;

        $req = "SELECT * FROM `{$this->table}` {$this->table_name} WHERE ".$this->where($atts);
        if(isset($atts['GROUP_BY'])){
            $req.=" GROUP BY ".$atts['GROUP_BY'];
        }
        if(isset($atts['ORDER_BY'])){
            $req.=" ORDER BY ".$atts['ORDER_BY'];
        }
        if(isset($atts['per_page']) && isset($atts['page'])){
            $req.=" LIMIT ".($atts['page']-1).",{$atts['per_page']}";
        }
        $results = $this->get_results($req);
        foreach ($results as $key => $item) {
            $results[$key] = $this->parse_item($item, $key);
        }
        $this->cache[$stat_name] = $results;
        return $this->cache[$stat_name];
    }

    function count($atts){
        $stat_name = 'count_'.md5(json_encode($atts));
        if (isset($this->cache[$stat_name])) {
            return $this->cache[$stat_name];
        }
        global $wpdb;
        $req = "SELECT COUNT({$this->table_name}.`id`) as nbr FROM `{$this->table}` {$this->table_name} WHERE ".$this->where($atts);

        $this->cache[$stat_name] = $this->get_results($req);
        return $this->cache[$stat_name];
    }

    function update($atts, $_where){
        $set = [];
        foreach ($atts as $key => $value) {
            if(!isset($this->attributes[$key])){
                continue;
            }
            $set[]= "`{$skey}`='{$value}";
        }
        $where = [];
        foreach ($_where as $key => $value) {
            if(!isset($this->attributes[$key])){
                continue;
            }
            $where[]= "`{$skey}`='{$value}";
        }
        $req = "UPDATE `{$this->table}` SET ".implode(', ', $set)." WHERE ".implode(' AND ', $where)."";
        return $this->query($req);
    }

    function insert($atts){
        $columns = [];
        $values = [];
        foreach ($atts as $key => $value) {
            if(!isset($this->attributes[$key])){
                continue;
            }
            $columns[]= "`{$skey}`";
            $values[]= "'{$value}'";
        }
        $req = "INSERT `{$this->table}` (".implode(', ', $set).") VALUES (".implode(' AND ', $where).")";
        return $this->query($req);
    }

    function delete($_where){
        $where = [];
        foreach ($_where as $key => $value) {
            $where[]= "`{$skey}`='{$value}";
        }
        $req = "DELETE FROM `{$this->table}` WHERE ".implode(' AND ', $where)."";
        return $this->query($req);
    }

    function query($query){
        return mysqli_query($this->DB, $query);
    }

    function get_results($query){
        $res = $this->query($query);
        if(!$res){
            return false;
        }
        $results = [];
        while ($item = mysqli_fetch_object($res)) {
            $results[] = $item;
        }
        return $results;
    }
}


class Model{
    public $id;
    public $_manager = null; // Model_Manager

    function __construct($atts=null){
        if($atts!=null){
            $reps = false;
            if(is_numeric($atts)){
                $reps = $this->getManager()->get(['id'=>$atts]);
            }
            if(is_array($reps) && isset($reps[0])){
                // Populate
                foreach($reps[0] as $key=>$value){
                    if(substr($value,0,2)=='a:'){
                        $value=unserialize($value);
                    }
                    $this->$key = $value;
                }
            }
            else{
                foreach($this->getManager()->attributes as $short=>$key){
                    $value = '';
                    if(isset($atts[$key])){
                        $value = $atts[$key];
                    }
                    $this->$key = $value;
                }
            }
        }
        return $this->populate();
    }

    function getManager(){
        $classname = get_class($this);
        return SYN()->$classname();
    }

    function populate(){
        return $this;
    }

    function save(){
        global $wpdb;
        $datas = array();

        foreach($this as $key=>$value){
            if(substr($key, 0, 1)=='_'){
                continue;
            }
            if(is_array($value)){
                $datas[$key]=serialize($value);
            }
            else{
                $datas[$key]=$value;
            }
        }

        if(!empty($this->id) && is_numeric($this->id) && $this->id>0){
            if(false != $this->getManager()->update( $datas, ['id'=>$this->id])){
                return $this->id;
            }
            SYN()->log('FAIL UPDATE', $this->table_name);
        }
        else{
            if(false != $insert_id = $this->getManager()->insert($datas)){
                return $insert_id;
            }
            SYN()->log('FAIL INSERT', $this->table_name);
        }
        return false;
    }

    function remove(){
        global $wpdb;
        if(is_numeric($this->id)){
            if(false !== $this->getManager()->delete(['id'=>$this->id])){
                return true;
            }
        }
        return false;
    }

}
