<?php

/**
 *
 * @param string $url
 * @return array
 */
function get_screenshot_route($url){
    global $eg;
    $date = date('Y-m-d');
    $image_name = md5($url) . '-' . $date . '.png';
    return array(
       'path' => ABS_PATH . 'img/screenshots/' . $image_name,
       'url' => $eg->base_url . '/img/screenshots/' . $image_name
    );
}

/**
 * save file from snapshot API
 * @param string $url
 * @param string $apikey
 * @param string $api_url
 * @return string
 */
function snapshot_call_p2i($url, $apikey, $api_url) {

    // URL can be those formats: http://www.google.com https://google.com google.com and www.google.com
    // But free rate plan does not support SSL link.
    //$url = "http://www.google.com";
    $device = 6;
    $size = "1024X768";
    // 0 - iPhone4, 1 - iPhone5, 2 - Android, 3 - WinPhone, 4 - iPad, 5 - Android Pad, 6 - Desktop
    $loop_flag = TRUE;
    $timeout = 120;
    // timeout after 120 seconds
    set_time_limit($timeout + 10);
    $start_time = time();
    $timeout_flag = false;

    $url = str_replace('https://', 'http://', $url);

    while ($loop_flag) {
        // We need call the API until we get the screenshot or error message
        try {
            $para = array(
                "p2i_url" => $url,
                "p2i_key" => $apikey,
                "p2i_device" => $device,
                "p2i_size" => $size
            );
            // connect page2images server

            $response = snapshot_connect($api_url, $para);
            //print_r($response);
            if (empty($response)) {
                $loop_flag = FALSE;
                // something error
                echo "something error " . $url;
                break;
            }
            else {
                $json_data = json_decode($response);
                if (empty($json_data->status)) {
                    $loop_flag = FALSE;
                    // api error
                    break;
                }
            }
            switch ($json_data->status) {
                case "error" :
                    // do something to handle error
                    $loop_flag = FALSE;
                    echo $json_data->errno . " " . $json_data->msg;
                    break;
                case "finished" :
                    // do something with finished. For example, show this image
                    // echo "<img src='$json_data->image_url'>";
                    $url = $json_data->image_url;
                    // Or you can download the image from our server
                    $route = get_screenshot_route($url);
                    $saved_folder = $route['path'];
                    $saved_file = $route['url'];
                    if (is_file($saved_folder)) {
                        return $saved_file;
                    }
                    if (file_put_contents($saved_folder, file_get_contents($url))) {
                        return $saved_file;
                    }
                    $loop_flag = FALSE;
                    //return 'http://'.$_SERVER_NAME.'/'.$saved_folder;
                    break;
                case "processing" :
                default :
                    if ((time() - $start_time) > $timeout) {
                        $loop_flag = false;
                        $timeout_flag = true;
                        // set the timeout flag. You can handle it later.
                    }
                    else {
                        sleep(3);
                        // This only work on windows.
                    }
                    break;
            }
        } catch (Exception $e) {
            // Do whatever you think is right to handle the exception.
            $loop_flag = FALSE;
            echo 'Caught exception: ', $e->getMessage(), "\n";
        }
    }

    if ($timeout_flag) {
        // handle the timeout event here
        echo "Error: Timeout after $timeout seconds.";
    }
    return '';
}

// curl to connect server
function snapshot_connect($url, $para) {
    if (empty($para)) {
        return false;
    }
    $ch = curl_init();
    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_HEADER, 0);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($para));
    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 30);
    curl_setopt($ch, CURLOPT_TIMEOUT, 30);
    $data = curl_exec($ch);
    curl_close($ch);
    return $data;
}

/**
 *
 * @param string $image_url
 * @param string $api_url
 * @param string $api_key
 * @return boolean | string
 */
function capture($image_url, $api_url, $api_key=''){
    $route = get_screenshot_route($image_url);
    $saved_folder = $route['path'];
    $saved_file = $route['url'];
    $file = file_get_contents($api_url.'?'.http_build_query(array(
        'type'=>'png',
        'url'=>$image_url,
        'key'=>$api_key,
        'crop'=>true,
        'screen_width'=>1024,
        'screen_height'=>600
        )));
    if($file && !empty($file) && file_put_contents($saved_folder, $file)) {
        $image_size = getimagesize($saved_folder);
        if($image_size[0]==1024){
            return $saved_file;
        }
        else{
            unlink($saved_folder);
        }
    }
    return false;
}

/**
 *
 * @global type $eg
 * @param string $url
 * @return boolean | string
 */
function screenshot($url) {
    global $eg;
    $saved_file = false;
    if (empty($eg->config['snapshot']['apikey']) && empty($eg->config['capture']['api_url'])) {
        return $saved_file;
    }
    if ($eg->config['capture']['api_url']) {
        $saved_file = capture($url, $eg->config['capture']['api_url'], $eg->config['capture']['apikey']);
    }
    elseif ($eg->config['snapshot']['apikey']) {
        $saved_file = snapshot_call_p2i($url, $eg->config['snapshot']['apikey'], $eg->config['snapshot']['api_url']);
    }
    return $saved_file;
}
