<?php

class SYN_Version_Manager {

    public $available;
    public $current;
    public $lastknown;
    public $path;

    function __construct() {
        $this->path = ABS_PATH . 'sql/upgrades';

        // Version manager is introduced in version 4.0
        $this->lastknown = 4.0;

        // Get local available version
        $this->available = $this->get_available();

        // Get current version
        $this->current = $this->get_current();

        header('SlideYourNet-Version: ' . $this->current);
    }

    function compile_version(string $version){
        $version = trim($version);
        $ver_format = substr_count($version, '.');
        if($ver_format === 1){
            $version = "{$version}.0";
        }
        $ver_part = explode('.', $version);
        $compiled_version = $ver_part[0].str_pad($ver_part[1], 2, "0", STR_PAD_LEFT).'.'.str_pad($ver_part[2], 3, "0", STR_PAD_LEFT);
        return (float) $compiled_version;
    }

    function get_available() {
        if (!is_file(ABS_PATH . 'VERSION')) {
            redirect(SYN()->current_url . '?message[]=3');
            exit;
        }
        return trim(file_get_contents(ABS_PATH . 'VERSION'));
    }

    function get_current() {
        $version = $this->lastknown;
        if (is_file(ABS_PATH . 'var/cache/CURRENT_VERSION')) {
            $version = trim(file_get_contents(ABS_PATH . 'var/cache/CURRENT_VERSION'));
        }
        else {
            $fp = fopen(ABS_PATH . 'var/cache/CURRENT_VERSION', 'w+');
            fwrite($fp, $version);
            fclose($fp);
        }
        return trim($version);
    }

    function set_current(string $version) {
        $fp = fopen(ABS_PATH . 'var/cache/CURRENT_VERSION', 'w+');
        if ($fp && fwrite($fp, $version)) {
            $this->current = $version;
            fclose($fp);
            return true;
        }
        fclose($fp);
        return false;
    }

    function get_upgrades(string $from = null, string $to = null) {
        if (!$from) {
            $from = $this->compile_version($this->current);
        }
        if (!$to) {
            $to = $this->compile_version($this->available);
        }
        $upgrades = [];

        $sql_upgrades = scandir($this->path);
        foreach ($sql_upgrades as $upgrade) {
            if (is_file($this->path . '/' . $upgrade) && substr($upgrade, -4) == '.sql') {
                $version_name = substr($upgrade, 0, strpos($upgrade, '.sql'));
                $up_version = $this->compile_version($version_name);
                if ($up_version > $from && $up_version <= $to) {
                    $upgrades[$version_name] = $up_version;
                }
            }
        }
        asort($upgrades);
        return $upgrades;
    }

    function do_upgrade(string $version){
        $query =$this->get_upgrade_query($version);
        debug(['Action'=>'Upgrade', 'version'=>$version, 'query'=>$query], 0, 1);
        if ($query && false !== mysqli_multi_query(SYN()->mysqli, $query)) {
            return $this->set_current($version);
        }
        return false;
    }

    function do_upgrades() {
        if ($this->current < $this->available) {
            $upgrades = $this->get_upgrades();
            if (count($upgrades)) {
                $updated = true;
                foreach ($upgrades as $version=>$compiled_version) {
                    if (!$this->do_upgrade($version)) {
                        $updated = false;
                    }
                }
                if (!$updated) {
                    echo mysqli_error(SYN()->mysqli);
                    // trigger error
                    header('SlideYourNet-UpgradeFailed: ' . $this->available);
                }
                return $updated;
            }
            debug(['Action'=>'Upgrade','info'=>'No upgrades'], 0, 1);
        }
        debug(['Action'=>'Upgrade','info'=>'Already up to date'], 0, 1);
    }

    function get_upgrade_query(string $version) {
        $filepath = $this->path . '/' . $version . '.sql';
        if (is_file($filepath)) {
            $root_query = file_get_contents($filepath);
            $query = str_replace(
                        array(
                            '{DB_PREFIX}'
                        ),
                        array(
                           SYN()->config['db_prefix']
                        ),
                        $root_query
                    );
            return $query;
        }
        return false;
    }

}
